module SPC_Tests (tests) where

import Control.Concurrent (threadDelay)
import Control.Monad (forM, forM_, replicateM)
import Data.IORef
import SPC
import GenServer
import Test.Tasty (TestTree, localOption, mkTimeout, testGroup)
import Test.Tasty.HUnit (assertFailure, testCase, (@?=))
import Data.Either (isRight)

tests :: TestTree
tests =
  localOption (mkTimeout 3000000) $
    testGroup
      "SPC (core)"
      [
        testCase "workerAdd" $ do
          spc <- startSPC
          w <- workerAdd spc "R2-D2"
          isRight w @?= True,
        testCase "workerAdd (2)" $ do
          spc <- startSPC
          w1 <- workerAdd spc "MSE-6"
          isRight w1 @?= True
          w2 <- workerAdd spc "GNK"
          isRight w2 @?= True,
        testCase "workerAdd (3)" $ do
          spc <- startSPC
          w1 <- workerAdd spc "C-3PO"
          isRight w1 @?= True
          w2 <- workerAdd spc "K-2SO"
          isRight w2 @?= True
          w3 <- workerAdd spc "IG-88"
          isRight w3 @?= True,
        testCase "workerAdd (fail)" $ do
          spc <- startSPC
          w1 <- workerAdd spc "BD-1"
          isRight w1 @?= True
          w2 <- workerAdd spc "BD-1"
          isRight w2 @?= False,
        testCase "Running a job" $ do
          ref <- newIORef False
          spc <- startSPC
          w <- workerAdd spc "R5-D4"
          isRight w @?= True
          j <- jobAdd spc $ Job (writeIORef ref True) 1
          r <- jobWait spc j
          r @?= Done
          x <- readIORef ref
          x @?= True
      ]
